/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.bc.sftp.authentication;

import org.ow2.petals.bc.sftp.authentication.exception.AuthenticationException;
import org.ow2.petals.bc.sftp.authentication.exception.PasswordMissingException;

import com.ebmwebsourcing.easycommons.lang.StringHelper;
import com.sshtools.j2ssh.authentication.PasswordAuthenticationClient;
import com.sshtools.j2ssh.authentication.SshAuthenticationClient;

/**
 * Utility class that holds the informations used by the component to perform a
 * user / password authentication onto the SSH server.
 * 
 * @author Charles Casadei - EBM WebSourcing
 */
public final class SshPasswordAuthentication extends SshAuthentication {

    /**
     * Password used for SSH user / password authentication.
     */
    private final String password;

    /**
     * Default constructor. Initialize the user and the password holded by an
     * instance of this class with the given values
     * 
     * @param user
     *            the username used for SSH user / password authentication
     * @param password
     *            the password used for SSH user / password authentication
     * @throws AuthenticationException
     *             if the configuration is not valid
     */
    public SshPasswordAuthentication(final String user, final String password)
            throws AuthenticationException {
        super(user);

        // Initialize the password with the given one
        this.password = password;

        // Validates the configuration
        this.validate();
    }

    /**
     * Accessor on the password used for SSH user / password authentication.
     * 
     * @return the password used for SSH user / password authentication
     */
    public final String getPassword() {
        return this.password;
    }

    /**
     * Builds the object used by the SFTPClientPool to perform the
     * authentication on the SSH server.
     * 
     * @return an object used by the SFTPClientPool to perform the
     *         authentication
     */
    @Override
    public final SshAuthenticationClient getAuthenticationClient() {
        PasswordAuthenticationClient client = new PasswordAuthenticationClient();
        client.setUsername(this.getUser());
        client.setPassword(password);

        return client;
    }

    /**
     * Builds & returns a portion of the key used by the SFTPClientPool to
     * identify a SSH session.
     * 
     * @return a portion of the key used by the SFTPClientPool
     */
    @Override
    public final String getKey() {
        return this.getUser() + password;
    }

    /**
     * Return the object content as String (Overrides the toString method).
     * 
     * @return the object content as String
     */
    @Override
    public final String toString() {
        return super.toString() + "password=" + this.password + ")";
    }

    /**
     * Checks if the given informations are OK.
     * 
     * @throws AuthenticationException
     *             if the configuration is not valid
     */
    private final void validate() throws AuthenticationException {
        // If there is a password
        if (StringHelper.isNullOrEmpty(this.password)) {
            throw new PasswordMissingException();
        }
    }

}
